/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.placeholder.filewriter;

import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;

import org.tinygroup.logger.Logger;
import org.tinygroup.logger.LoggerFactory;
import org.tinygroup.placeholder.FileWriter;
import org.tinygroup.vfs.FileObject;

public class JarFileWriter implements FileWriter {

	private static final String SCHEMA_PATH = "///";
	private static final String FILE_SCHEMA = "jar";
	private static final Logger logger = LoggerFactory
			.getLogger(JarFileWriter.class);
	private String schema = FILE_SCHEMA;

	public void setSchema(String schema) {
		this.schema = schema;
	}

	public String getSchema() {
		return FILE_SCHEMA;
	}

	/**
	 * 根据给出的jar包文件路径，替换文件内容.
	 */
	public void writeFile(FileObject fileObject, String content, String encoding) {

		// 获得jar包路径
		String jarPath = fileObject.getAbsolutePath();
		jarPath = jarPath.substring(
				jarPath.indexOf(SCHEMA_PATH) + SCHEMA_PATH.length(),
				jarPath.length() - 2);
		// String targetPath = fileObject.getName().getBaseName();
		// 获得目标文件相对路径
		String targetPath = fileObject.getAbsolutePath();
		// 截取jar!之后的部分
		targetPath = targetPath.substring(targetPath.indexOf("!") + 1);
		// 准备创建临时文件，获得项目classes路径
		String resourcePath = Thread.currentThread().getContextClassLoader()
				.getResource("").toString();
		String targetRootPath = resourcePath.substring(resourcePath
				.indexOf("file:") + 5);
		// 临时文件全路径
		String TempFilePath = targetRootPath + targetPath;
		File tempFile = null;
		try {
			// 创建临时文件
			tempFile = write(content, TempFilePath, encoding);
			// 关闭VFS文件系统，否则在win系统下后面的jar操作会因为文件句柄问题失败
			// VFS.getManager().closeFileSystem(fileObject.getFileSystem());
		} catch (Exception e) {
			logger.errorMessage(e.getMessage(), e);
		}
		// 替换jar包文件操作
		ZipTools.addFileToExistingZip("\"" + targetRootPath + "\"", "\""
				+ targetPath + "\"", "\"" + jarPath + "\"");
		// 删除临时文件
		tempFile.delete();
	}

	/**
	 * 根据内容创建临时文件及路径
	 * 
	 * @param fileContent
	 * @param fileName
	 * @param encoding
	 * @return
	 * @throws Exception
	 */
	public File write(String fileContent, String fileName, String encoding)
			throws Exception {
		FileOutputStream fos = null;
		OutputStreamWriter osw = null;
		File tempFile = new File(fileName);
		try {
			if (!tempFile.getParentFile().exists())
				tempFile.getParentFile().mkdirs();
			if (!tempFile.exists())
				tempFile.createNewFile();
			fos = new FileOutputStream(fileName);
			osw = new OutputStreamWriter(fos, encoding);
			osw.write(fileContent);
			osw.flush();
		} catch (Exception e) {
			logger.errorMessage(e.getMessage(), e);
		} finally {
			if (fos != null)
				fos.close();
			if (osw != null)
				osw.close();
		}
		return tempFile;
	}

}
